@@ -59,7 +59,7 @@ If you want to know how some parts works, check tests/visual where are displayed
 from . import analyze
 from . import best_params
 from . import models
-from . import config
+from . import configuration
 from . import data_preprocessing
 from . import database
 from . import define_inputs
@@ -67,13 +67,14 @@ from . import evaluate_predictions
 from . import misc
 from . import plot
 from . import test_data
+from . import main_loop
 
 from . import main
 
 
-__version__ = "1.34"
+__version__ = "1.36"
 __author__ = "Daniel Malachov"
 __license__ = "MIT"
 __email__ = "malachovd@seznam.cz"
 
-__all__ = ['analyze', 'best_params', 'config', 'data_preprocessing', 'database', 'define_inputs', 'evaluate_predictions', 'main', 'misc', 'models', 'plot', 'test_data']
+__all__ = ['analyze', 'best_params', 'configuration', 'data_preprocessing', 'database', 'define_inputs', 'evaluate_predictions', 'main', 'misc', 'models', 'plot', 'test_data', 'main_loop']
